#define PARSER_NL                               1
#define PARSER_PREAMBLE                         2
#define PARSER_COLONSP                          3
#define PARSER_STRING                           4
#define PARSER_PACKAGE                          5
#define PARSER_REQUEST                          6
#define PARSER_FEEDBACK_BOOL                    7
#define PARSER_FEEDBACK_IDENT                   8
#define PARSER_FEEDBACK_ENUM                    9
#define PARSER_FEEDBACK_INT                    10
#define PARSER_FEEDBACK_NAT                    11
#define PARSER_FEEDBACK_POSINT                 12
#define PARSER_FEEDBACK_PKGNAME                13
#define PARSER_FEEDBACK_TYPEDECL               14
#define PARSER_FEEDBACK_VPKG                   15
#define PARSER_FEEDBACK_VEQPKG                 16
#define PARSER_FEEDBACK_VPKGFORMULA            17
#define PARSER_FEEDBACK_VPKGLIST               18
#define PARSER_FEEDBACK_VEQPKGLIST             19
#define PARSER_FEEDBACK_STRING                 20
#define PARSER_TRUE                            21
#define PARSER_FALSE                           22
#define PARSER_IDENT                           23
#define PARSER_TYPE_BOOL                       24
#define PARSER_TYPE_INT                        25
#define PARSER_TYPE_NAT                        26
#define PARSER_TYPE_POSINT                     27
#define PARSER_TYPE_STRING                     28
#define PARSER_TYPE_PKGNAME                    29
#define PARSER_TYPE_IDENT                      30
#define PARSER_TYPE_VPKG                       31
#define PARSER_TYPE_VEQPKG                     32
#define PARSER_TYPE_VPKGFORMULA                33
#define PARSER_TYPE_VPKGLIST                   34
#define PARSER_TYPE_VEQPKGLIST                 35
#define PARSER_TYPE_ENUM                       36
#define PARSER_PKGNAME                         37
#define PARSER_POSINT                          38
#define PARSER_NAT                             39
#define PARSER_INT                             40
#define PARSER_EQUAL                           41
#define PARSER_RELOP                           42
#define PARSER_BAR                             43
#define PARSER_COMMA                           44
#define PARSER_TRUEX                           45
#define PARSER_FALSEX                          46
#define PARSER_COLON                           47
#define PARSER_LBRAC                           48
#define PARSER_RBRAC                           49
#define PARSER_QUOTED                          50
