/* Generated by re2c 1.3 on Mon Jan 11 13:39:53 2021 */
#line 1 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
// {{{ MIT License

// Copyright 2017 Roland Kaminski

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

// }}}

#line 69 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"


int Parser::lex() {
    for(;;) {
        start();
        
#line 35 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
{
	YYCTYPE yych;
	if ((YYLIMIT - YYCURSOR) < 12) YYFILL(12);
	yych = *YYCURSOR;
	switch (yych) {
	case 0x00:	goto yy2;
	case '\t':
	case ' ':	goto yy6;
	case '\n':
	case '\r':	goto yy8;
	case '!':	goto yy11;
	case '"':	goto yy12;
	case '#':	goto yy13;
	case '%':
	case '(':
	case ')':
	case '.':
	case '/':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':	goto yy16;
	case '+':	goto yy19;
	case ',':	goto yy20;
	case '-':	goto yy22;
	case '0':	goto yy23;
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy26;
	case ':':	goto yy29;
	case '<':	goto yy31;
	case '=':	goto yy33;
	case '>':	goto yy35;
	case '[':	goto yy37;
	case ']':	goto yy39;
	case 'a':
	case 'c':
	case 'd':
	case 'g':
	case 'h':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'o':
	case 'q':
	case 'u':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy41;
	case 'b':	goto yy44;
	case 'e':	goto yy45;
	case 'f':	goto yy46;
	case 'i':	goto yy47;
	case 'n':	goto yy48;
	case 'p':	goto yy49;
	case 'r':	goto yy50;
	case 's':	goto yy51;
	case 't':	goto yy52;
	case 'v':	goto yy53;
	case '|':	goto yy54;
	default:	goto yy4;
	}
yy2:
	++YYCURSOR;
#line 75 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return 0; }
#line 133 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy4:
	++YYCURSOR;
yy5:
#line 117 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ syntaxError(); }
#line 139 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy6:
	++YYCURSOR;
#line 92 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ continue; }
#line 144 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy8:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	switch (yych) {
	case '\n':
	case '\r':	goto yy8;
	default:	goto yy10;
	}
yy10:
#line 76 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_NL; }
#line 157 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy11:
	yych = *++YYCURSOR;
	switch (yych) {
	case '=':	goto yy56;
	default:	goto yy5;
	}
yy12:
	yych = *(YYMARKER = ++YYCURSOR);
	switch (yych) {
	case '\n':
	case '\r':	goto yy5;
	default:	goto yy59;
	}
yy13:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	switch (yych) {
	case '\n':	goto yy15;
	default:	goto yy13;
	}
yy15:
#line 81 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_NL; }
#line 182 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy16:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
yy17:
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy16;
	default:	goto yy18;
	}
yy18:
#line 116 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_PKGNAME; }
#line 264 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy19:
	yych = *++YYCURSOR;
	switch (yych) {
	case '0':	goto yy23;
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy26;
	default:	goto yy17;
	}
yy20:
	++YYCURSOR;
#line 82 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_COMMA; }
#line 284 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy22:
	yych = *++YYCURSOR;
	switch (yych) {
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy64;
	default:	goto yy17;
	}
yy23:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy16;
	case '0':	goto yy23;
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy26;
	default:	goto yy25;
	}
yy25:
#line 113 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_NAT; }
#line 380 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy26:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy16;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy26;
	default:	goto yy28;
	}
yy28:
#line 112 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_POSINT; }
#line 461 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy29:
	yych = *++YYCURSOR;
	switch (yych) {
	case ' ':	goto yy67;
	default:	goto yy30;
	}
yy30:
#line 78 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_COLON; }
#line 471 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy31:
	yych = *++YYCURSOR;
	switch (yych) {
	case '=':	goto yy69;
	default:	goto yy32;
	}
yy32:
#line 89 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = LT; return PARSER_RELOP; }
#line 481 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy33:
	++YYCURSOR;
#line 84 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = EQ; return PARSER_EQUAL; }
#line 486 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy35:
	yych = *++YYCURSOR;
	switch (yych) {
	case '=':	goto yy71;
	default:	goto yy36;
	}
yy36:
#line 87 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = GT; return PARSER_RELOP; }
#line 496 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy37:
	++YYCURSOR;
#line 90 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_LBRAC; }
#line 501 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy39:
	++YYCURSOR;
#line 91 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_RBRAC; }
#line 506 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy41:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
yy42:
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '.':
	case '/':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':	goto yy16;
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy41;
	default:	goto yy43;
	}
yy43:
#line 115 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_IDENT; }
#line 588 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy44:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy73;
	default:	goto yy42;
	}
yy45:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy74;
	default:	goto yy42;
	}
yy46:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy75;
	default:	goto yy42;
	}
yy47:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'd':	goto yy76;
	case 'n':	goto yy77;
	default:	goto yy42;
	}
yy48:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy78;
	default:	goto yy42;
	}
yy49:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy79;
	case 'k':	goto yy80;
	case 'o':	goto yy81;
	case 'r':	goto yy82;
	default:	goto yy42;
	}
yy50:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy83;
	default:	goto yy42;
	}
yy51:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy84;
	default:	goto yy42;
	}
yy52:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'r':	goto yy85;
	default:	goto yy42;
	}
yy53:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy86;
	case 'p':	goto yy87;
	default:	goto yy42;
	}
yy54:
	++YYCURSOR;
#line 83 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_BAR; }
#line 658 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy56:
	++YYCURSOR;
#line 85 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = NEQ; return PARSER_RELOP; }
#line 663 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy58:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
yy59:
	switch (yych) {
	case '\n':
	case '\r':	goto yy60;
	case '"':	goto yy61;
	case '\\':	goto yy63;
	default:	goto yy58;
	}
yy60:
	YYCURSOR = YYMARKER;
	goto yy5;
yy61:
	++YYCURSOR;
#line 93 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string(1, 1)); return PARSER_QUOTED; }
#line 683 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy63:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	switch (yych) {
	case '\n':
	case '\r':	goto yy60;
	default:	goto yy58;
	}
yy64:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy16;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy64;
	default:	goto yy66;
	}
yy66:
#line 114 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_INT; }
#line 773 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy67:
	++YYCURSOR;
#line 77 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_COLONSP; }
#line 778 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy69:
	++YYCURSOR;
#line 88 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = LE; return PARSER_RELOP; }
#line 783 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy71:
	++YYCURSOR;
#line 86 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = GE; return PARSER_RELOP; }
#line 788 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy73:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy88;
	default:	goto yy42;
	}
yy74:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy89;
	default:	goto yy42;
	}
yy75:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy90;
	default:	goto yy42;
	}
yy76:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy91;
	default:	goto yy42;
	}
yy77:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy92;
	default:	goto yy42;
	}
yy78:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy94;
	default:	goto yy42;
	}
yy79:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'c':	goto yy96;
	default:	goto yy42;
	}
yy80:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'g':	goto yy97;
	default:	goto yy42;
	}
yy81:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy98;
	default:	goto yy42;
	}
yy82:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy99;
	default:	goto yy42;
	}
yy83:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'q':	goto yy100;
	default:	goto yy42;
	}
yy84:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'r':	goto yy101;
	default:	goto yy42;
	}
yy85:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy102;
	default:	goto yy42;
	}
yy86:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'q':	goto yy103;
	default:	goto yy42;
	}
yy87:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'k':	goto yy104;
	default:	goto yy42;
	}
yy88:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy105;
	default:	goto yy42;
	}
yy89:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy107;
	default:	goto yy42;
	}
yy90:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy109;
	default:	goto yy42;
	}
yy91:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy110;
	default:	goto yy42;
	}
yy92:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy93;
	}
yy93:
#line 98 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_INT; }
#line 981 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy94:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy95;
	}
yy95:
#line 99 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_NAT; }
#line 1060 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy96:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'k':	goto yy111;
	default:	goto yy42;
	}
yy97:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy112;
	default:	goto yy42;
	}
yy98:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'i':	goto yy113;
	default:	goto yy42;
	}
yy99:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy114;
	default:	goto yy42;
	}
yy100:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy115;
	default:	goto yy42;
	}
yy101:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'i':	goto yy116;
	default:	goto yy42;
	}
yy102:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy117;
	default:	goto yy42;
	}
yy103:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'p':	goto yy119;
	default:	goto yy42;
	}
yy104:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'g':	goto yy120;
	default:	goto yy42;
	}
yy105:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy106;
	}
yy106:
#line 97 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_BOOL; }
#line 1193 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy107:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy108;
	}
yy108:
#line 109 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_ENUM; }
#line 1272 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy109:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy122;
	default:	goto yy42;
	}
yy110:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy124;
	default:	goto yy42;
	}
yy111:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy126;
	default:	goto yy42;
	}
yy112:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy127;
	default:	goto yy42;
	}
yy113:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy128;
	default:	goto yy42;
	}
yy114:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy129;
	default:	goto yy42;
	}
yy115:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy130;
	default:	goto yy42;
	}
yy116:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy131;
	default:	goto yy42;
	}
yy117:
	yych = *++YYCURSOR;
	switch (yych) {
	case '!':	goto yy132;
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy118;
	}
yy118:
#line 110 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TRUE; }
#line 1400 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy119:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'k':	goto yy134;
	default:	goto yy42;
	}
yy120:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	case 'f':	goto yy135;
	case 'l':	goto yy136;
	default:	goto yy121;
	}
yy121:
#line 104 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_VPKG; }
#line 1485 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy122:
	yych = *++YYCURSOR;
	switch (yych) {
	case '!':	goto yy137;
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy123;
	}
yy123:
#line 111 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_FALSE; }
#line 1565 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy124:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy125;
	}
yy125:
#line 103 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_IDENT; }
#line 1644 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy126:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'g':	goto yy139;
	default:	goto yy42;
	}
yy127:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy140;
	default:	goto yy42;
	}
yy128:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy141;
	default:	goto yy42;
	}
yy129:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'b':	goto yy143;
	default:	goto yy42;
	}
yy130:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy144;
	default:	goto yy42;
	}
yy131:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'g':	goto yy145;
	default:	goto yy42;
	}
yy132:
	++YYCURSOR;
#line 79 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_TRUEX; }
#line 1685 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy134:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'g':	goto yy147;
	default:	goto yy42;
	}
yy135:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy149;
	default:	goto yy42;
	}
yy136:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'i':	goto yy150;
	default:	goto yy42;
	}
yy137:
	++YYCURSOR;
#line 80 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ return PARSER_FALSEX; }
#line 1708 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy139:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy151;
	default:	goto yy42;
	}
yy140:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy153;
	default:	goto yy42;
	}
yy141:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy142;
	}
yy142:
#line 100 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_POSINT; }
#line 1799 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy143:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy155;
	default:	goto yy42;
	}
yy144:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy156;
	default:	goto yy42;
	}
yy145:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy146;
	}
yy146:
#line 101 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_STRING; }
#line 1890 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy147:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	case 'l':	goto yy158;
	default:	goto yy148;
	}
yy148:
#line 105 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_VEQPKG; }
#line 1969 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy149:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'r':	goto yy159;
	default:	goto yy42;
	}
yy150:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy160;
	default:	goto yy42;
	}
yy151:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy152;
	}
yy152:
#line 95 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_PACKAGE; }
#line 2060 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy153:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy154;
	}
yy154:
#line 102 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_PKGNAME; }
#line 2139 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy155:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy161;
	default:	goto yy42;
	}
yy156:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy157;
	}
yy157:
#line 96 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_REQUEST; }
#line 2224 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy158:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'i':	goto yy163;
	default:	goto yy42;
	}
yy159:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy164;
	default:	goto yy42;
	}
yy160:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy165;
	default:	goto yy42;
	}
yy161:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy162;
	}
yy162:
#line 94 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_PREAMBLE; }
#line 2321 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy163:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy167;
	default:	goto yy42;
	}
yy164:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy168;
	default:	goto yy42;
	}
yy165:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy166;
	}
yy166:
#line 107 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_VPKGLIST; }
#line 2412 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy167:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy169;
	default:	goto yy42;
	}
yy168:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy171;
	default:	goto yy42;
	}
yy169:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy170;
	}
yy170:
#line 108 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_VEQPKGLIST; }
#line 2503 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy171:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy172;
	default:	goto yy42;
	}
yy172:
	yych = *++YYCURSOR;
	switch (yych) {
	case '%':
	case '(':
	case ')':
	case '+':
	case '-':
	case '.':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '@':
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy42;
	default:	goto yy173;
	}
yy173:
#line 106 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ token_.index = dep_.index(string()); return PARSER_TYPE_VPKGFORMULA; }
#line 2588 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
}
#line 118 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"

    }
    return 0;
}

int Parser::lexString() {
    start();
start:
    
#line 2600 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
{
	YYCTYPE yych;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	switch (yych) {
	case '\n':
	case '\r':	goto yy178;
	default:	goto yy176;
	}
yy176:
	++YYCURSOR;
#line 127 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ goto start; }
#line 2614 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
yy178:
	++YYCURSOR;
#line 128 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"
	{ unget(); token_.index = dep_.index(string()); return PARSER_STRING; }
#line 2619 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/lexer.hh"
}
#line 130 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/lexer.xh"

    return 0;
}
