/* Generated by re2c 1.3 on Mon Jan 11 13:39:53 2021 */
#line 1 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
// {{{ MIT License

// Copyright 2017 Roland Kaminski

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

// }}}

#line 31 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"


int CritParser::lex() {
    for(;;) {
        start();
        
#line 35 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
{
	YYCTYPE yych;
	if ((YYLIMIT - YYCURSOR) < 16) YYFILL(16);
	yych = *YYCURSOR;
	switch (yych) {
	case 0x00:	goto yy2;
	case '(':	goto yy6;
	case ')':	goto yy8;
	case '+':	goto yy10;
	case ',':	goto yy12;
	case '-':	goto yy14;
	case 'a':	goto yy16;
	case 'b':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'o':
	case 'p':
	case 'q':
	case 't':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	case 'c':	goto yy20;
	case 'd':	goto yy21;
	case 'i':	goto yy22;
	case 'n':	goto yy23;
	case 'r':	goto yy24;
	case 's':	goto yy25;
	case 'u':	goto yy26;
	default:	goto yy4;
	}
yy2:
	++YYCURSOR;
#line 37 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return 0; }
#line 79 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy4:
	++YYCURSOR;
#line 58 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ syntaxError(); return 0; }
#line 84 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy6:
	++YYCURSOR;
#line 41 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_LPAREN; }
#line 89 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy8:
	++YYCURSOR;
#line 42 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_RPAREN; }
#line 94 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy10:
	++YYCURSOR;
#line 40 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ token_.maximize = true;  return PARSER_SIGN; }
#line 99 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy12:
	++YYCURSOR;
#line 38 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_COMMA; }
#line 104 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy14:
	++YYCURSOR;
#line 39 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ token_.maximize = false; return PARSER_SIGN; }
#line 109 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy16:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy27;
	default:	goto yy19;
	}
yy17:
#line 57 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ token_.string = &string(string()); return PARSER_ATTR; }
#line 119 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy18:
	++YYCURSOR;
	if (YYLIMIT <= YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
yy19:
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy17;
	}
yy20:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'h':	goto yy28;
	case 'o':	goto yy29;
	default:	goto yy19;
	}
yy21:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy30;
	default:	goto yy19;
	}
yy22:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy31;
	default:	goto yy19;
	}
yy23:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy32;
	case 'o':	goto yy33;
	default:	goto yy19;
	}
yy24:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy34;
	default:	goto yy19;
	}
yy25:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy35;
	case 'u':	goto yy36;
	default:	goto yy19;
	}
yy26:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy37;
	case 'p':	goto yy38;
	default:	goto yy19;
	}
yy27:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'i':	goto yy40;
	default:	goto yy19;
	}
yy28:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy41;
	default:	goto yy19;
	}
yy29:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy42;
	default:	goto yy19;
	}
yy30:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'w':	goto yy43;
	default:	goto yy19;
	}
yy31:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy44;
	default:	goto yy19;
	}
yy32:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'w':	goto yy45;
	default:	goto yy19;
	}
yy33:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy47;
	default:	goto yy19;
	}
yy34:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy48;
	case 'q':	goto yy49;
	default:	goto yy19;
	}
yy35:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy50;
	default:	goto yy19;
	}
yy36:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy51;
	default:	goto yy19;
	}
yy37:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy53;
	default:	goto yy19;
	}
yy38:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	case 'g':	goto yy54;
	default:	goto yy39;
	}
yy39:
#line 47 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_UP; }
#line 323 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy40:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'g':	goto yy55;
	default:	goto yy19;
	}
yy41:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy56;
	default:	goto yy19;
	}
yy42:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy57;
	default:	goto yy19;
	}
yy43:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy58;
	default:	goto yy19;
	}
yy44:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy60;
	default:	goto yy19;
	}
yy45:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy46;
	}
yy46:
#line 45 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_NEW; }
#line 399 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy47:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy61;
	default:	goto yy19;
	}
yy48:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy62;
	default:	goto yy19;
	}
yy49:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy63;
	default:	goto yy19;
	}
yy50:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy64;
	default:	goto yy19;
	}
yy51:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy52;
	}
yy52:
#line 53 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_SUM; }
#line 469 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy53:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy65;
	default:	goto yy19;
	}
yy54:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'r':	goto yy66;
	default:	goto yy19;
	}
yy55:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy67;
	default:	goto yy19;
	}
yy56:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'g':	goto yy68;
	default:	goto yy19;
	}
yy57:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy69;
	default:	goto yy19;
	}
yy58:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy59;
	}
yy59:
#line 48 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_DOWN; }
#line 545 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy60:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy71;
	default:	goto yy19;
	}
yy61:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'p':	goto yy72;
	default:	goto yy19;
	}
yy62:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'v':	goto yy73;
	default:	goto yy19;
	}
yy63:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy74;
	default:	goto yy19;
	}
yy64:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy75;
	default:	goto yy19;
	}
yy65:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy76;
	default:	goto yy19;
	}
yy66:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy77;
	default:	goto yy19;
	}
yy67:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy78;
	default:	goto yy19;
	}
yy68:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy79;
	default:	goto yy19;
	}
yy69:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy70;
	}
yy70:
#line 52 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_COUNT; }
#line 645 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy71:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy80;
	default:	goto yy19;
	}
yy72:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy81;
	default:	goto yy19;
	}
yy73:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy82;
	default:	goto yy19;
	}
yy74:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy83;
	default:	goto yy19;
	}
yy75:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'i':	goto yy84;
	default:	goto yy19;
	}
yy76:
	yych = *(YYMARKER = ++YYCURSOR);
	switch (yych) {
	case '_':	goto yy85;
	default:	goto yy19;
	}
yy77:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'd':	goto yy87;
	default:	goto yy19;
	}
yy78:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'd':	goto yy88;
	default:	goto yy19;
	}
yy79:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'd':	goto yy90;
	default:	goto yy19;
	}
yy80:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'l':	goto yy92;
	default:	goto yy19;
	}
yy81:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy93;
	default:	goto yy19;
	}
yy82:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'd':	goto yy94;
	default:	goto yy19;
	}
yy83:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy96;
	default:	goto yy19;
	}
yy84:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy98;
	default:	goto yy19;
	}
yy85:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'r':	goto yy99;
	default:	goto yy86;
	}
yy86:
	YYCURSOR = YYMARKER;
	goto yy17;
yy87:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy100;
	default:	goto yy19;
	}
yy88:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy89;
	}
yy89:
#line 56 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_ALIGNED; }
#line 790 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy90:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy91;
	}
yy91:
#line 44 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_CHANGED; }
#line 836 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy92:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'r':	goto yy101;
	default:	goto yy19;
	}
yy93:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'd':	goto yy102;
	default:	goto yy19;
	}
yy94:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy95;
	}
yy95:
#line 46 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_REMOVED; }
#line 894 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy96:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy97;
	}
yy97:
#line 51 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_REQUEST; }
#line 940 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy98:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy103;
	default:	goto yy19;
	}
yy99:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy105;
	default:	goto yy86;
	}
yy100:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'r':	goto yy106;
	default:	goto yy19;
	}
yy101:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy107;
	default:	goto yy19;
	}
yy102:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'a':	goto yy108;
	default:	goto yy19;
	}
yy103:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy104;
	}
yy104:
#line 43 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_SOLUTION; }
#line 1016 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy105:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'c':	goto yy109;
	default:	goto yy86;
	}
yy106:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy110;
	default:	goto yy19;
	}
yy107:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'q':	goto yy111;
	default:	goto yy19;
	}
yy108:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy112;
	default:	goto yy19;
	}
yy109:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'o':	goto yy113;
	default:	goto yy86;
	}
yy110:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'q':	goto yy114;
	default:	goto yy19;
	}
yy111:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy115;
	default:	goto yy19;
	}
yy112:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy116;
	default:	goto yy19;
	}
yy113:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy118;
	default:	goto yy86;
	}
yy114:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'u':	goto yy119;
	default:	goto yy19;
	}
yy115:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy120;
	default:	goto yy19;
	}
yy116:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy117;
	}
yy117:
#line 54 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_NOTUPTODATE; }
#line 1128 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy118:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'm':	goto yy121;
	default:	goto yy86;
	}
yy119:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy122;
	default:	goto yy19;
	}
yy120:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy123;
	default:	goto yy19;
	}
yy121:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'e':	goto yy124;
	default:	goto yy86;
	}
yy122:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy125;
	default:	goto yy19;
	}
yy123:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy126;
	default:	goto yy19;
	}
yy124:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'n':	goto yy128;
	default:	goto yy86;
	}
yy125:
	yych = *++YYCURSOR;
	switch (yych) {
	case 't':	goto yy129;
	default:	goto yy19;
	}
yy126:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy127;
	}
yy127:
#line 49 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_INSTALLREQUEST; }
#line 1222 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy128:
	yych = *++YYCURSOR;
	switch (yych) {
	case 'd':	goto yy131;
	default:	goto yy86;
	}
yy129:
	yych = *++YYCURSOR;
	switch (yych) {
	case '-':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'a':
	case 'b':
	case 'c':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'l':
	case 'm':
	case 'n':
	case 'o':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 't':
	case 'u':
	case 'v':
	case 'w':
	case 'x':
	case 'y':
	case 'z':	goto yy18;
	default:	goto yy130;
	}
yy130:
#line 50 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_UPGRADEREQUEST; }
#line 1274 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
yy131:
	yych = *++YYCURSOR;
	switch (yych) {
	case 's':	goto yy132;
	default:	goto yy86;
	}
yy132:
	++YYCURSOR;
#line 55 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"
	{ return PARSER_UNSAT_RECOMMENDS; }
#line 1285 "/home/kaminski/Documents/git/potassco/aspcud/build/debug/libcudf//src/critlexer.hh"
}
#line 59 "/home/kaminski/Documents/git/potassco/aspcud/libcudf//src/critlexer.xh"

    }
    return 0;
}
