#define PARSER_COMMA                            1
#define PARSER_SIGN                             2
#define PARSER_COUNT                            3
#define PARSER_LPAREN                           4
#define PARSER_RPAREN                           5
#define PARSER_NOTUPTODATE                      6
#define PARSER_UNSAT_RECOMMENDS                 7
#define PARSER_SUM                              8
#define PARSER_ALIGNED                          9
#define PARSER_NEW                             10
#define PARSER_REMOVED                         11
#define PARSER_CHANGED                         12
#define PARSER_SOLUTION                        13
#define PARSER_UP                              14
#define PARSER_DOWN                            15
#define PARSER_INSTALLREQUEST                  16
#define PARSER_UPGRADEREQUEST                  17
#define PARSER_REQUEST                         18
#define PARSER_ATTR                            19
